@@ -30,30 +30,6 @@
 #include <sys/stat.h>
 #include "fig2dev.h"
 
-/* define PATH_MAX if not already defined */
-/* taken from the X11R5 server/os/osfonts.c file */
-#ifndef X_NOT_POSIX
-#ifdef _POSIX_SOURCE
-#include <limits.h>
-#else
-#if !defined(sun) || defined(sparc)
-#define _POSIX_SOURCE
-#include <limits.h>
-#undef _POSIX_SOURCE
-#endif /* !defined(sun) || defined(sparc) */
-#endif /* _POSIX_SOURCE */
-#endif /* X_NOT_POSIX */
-
-#ifndef PATH_MAX
-#include <sys/param.h>
-#ifdef MAXPATHLEN
-#define PATH_MAX MAXPATHLEN
-#else
-#define PATH_MAX 1024
-#endif /* MAXPATHLEN */
-#endif /* PATH_MAX */
-
-
 FILE *
 open_picfile(name, type)
     char	*name;
@@ -68,12 +44,12 @@ open_picfile(name, type)
     compname = NULL;
     /* see if the filename ends with .Z */
     /* if so, generate uncompress command and use pipe (filetype = 1) */
-    if (strlen(name) > 2 && !strcmp(".Z", name + (strlen(name)-2))) {
+    if (strlen(name) > (size_t)2 && !strcmp(".Z", name + (strlen(name)-2))) {
 	sprintf(unc,"uncompress -c %s",name);
 	*type = 1;
     /* or with .z or .gz */
-    } else if ((strlen(name) > 3 && !strcmp(".gz", name + (strlen(name)-3))) ||
-	      ((strlen(name) > 2 && !strcmp(".z", name + (strlen(name)-2))))) {
+    } else if ((strlen(name) > (size_t)3 && !strcmp(".gz", name + (strlen(name)-3))) ||
+	      ((strlen(name) > (size_t)2 && !strcmp(".z", name + (strlen(name)-2))))) {
 	sprintf(unc,"gunzip -qc %s",name);
 	*type = 1;
     /* none of the above, see if the file with .Z or .gz or .z appended exists */
